/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

import com.sun.lwuit.*;
import com.sun.lwuit.events.ActionEvent;
import com.sun.lwuit.events.ActionListener;
import com.sun.lwuit.layouts.BorderLayout;
import com.sun.lwuit.layouts.BoxLayout;
import com.sun.lwuit.list.DefaultListModel;
import com.sun.lwuit.list.ListCellRenderer;
import com.sun.lwuit.plaf.Border;
import com.sun.lwuit.plaf.UIManager;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;
import javax.microedition.midlet.MIDlet;
import org.diacalc.mobile.components.*;
import org.diacalc.mobile.maths.*;
import org.diacalc.mobile.products.ProductInMenu;


public class DiaryForm extends FormBase implements ActionListener{
    private static final int N_EV_COMMAND = 30;
    private static final int E_EV_COMMAND = 31;
    private static final int D_EV_COMMAND = 32;
    private static final int FILL_MENU_COMMAND = 33;
    private static final int N_COM_COMMAND = 34;
    private static final int EXPORT_DIARY_COMMAND = 35;
    private static final int GET_INFO_PER_DAY = 36;

    private static final int YES_COMMAND = 37;
    private static final int NO_COMMAND = 38;

    private static final Command newEventCommand = new Command("Новый СК", N_EV_COMMAND);
    private static final Command editEventCommand = new Command("Изм. запись", E_EV_COMMAND);
    private static final Command delEventCommand = new Command("Удал. запись", D_EV_COMMAND);
    private static final Command fillMenuCommand = new Command("Запол. меню", FILL_MENU_COMMAND);
    private static final Command newCommentCommand = new Command("Нов. коммент.", N_COM_COMMAND);
    private static final Command exportDiaryCommand = new Command("Экспорт.", EXPORT_DIARY_COMMAND);
    private static final Command getInfoPerDayCommand = new Command("Инф. за день", GET_INFO_PER_DAY);

    private static final Command yesCommand = new Command("Да",YES_COMMAND);
    private static final Command noCommand = new Command("Нет",NO_COMMAND);


    private DCJmobile master;
    private DefaultListModel model;
    private List diaryL;

    public String getIconName(){
        return "diary";
    }
    public void actionPerformed(ActionEvent ev){
        int id = ev.getCommand().getId();
        switch(id){
            case E_EV_COMMAND: editEvent(); break;
            case D_EV_COMMAND: deleteEvent(); break;
            case N_EV_COMMAND: addEvent(); break;
            case FILL_MENU_COMMAND: fillMenu(); break;
            case N_COM_COMMAND: addComment(); break;
            case EXPORT_DIARY_COMMAND: exportDiary(); break;
            case GET_INFO_PER_DAY: getInfoPerDay(); break;
        }
    }
    
    private Hashtable shRes = null;
    private Hashtable commRes = null;
    private Hashtable menuRes = null;
    
    private void exportDiary(){
        if (diaryL.getModel().getSize()==0) return;
        Hashtable ver_check = null;
        
        final InfoDialog infoDlg = new InfoDialog();
        Display.getInstance().callSeriallyAndWait(new Runnable(){
            public void run(){
                infoDlg.showModeless();
            }
        });
        //infoDlg.showModeless();
        //А сама загрузка должна быть в отдельном потоке
        final Vector sugars  = new Vector();
        final Vector menus = new Vector();
        final Vector comments = new Vector();

        for(int i=0;i<master.getDiary().size();i++){
            switch (((DiaryUnit)master.getDiary().elementAt(i)).getType() ){
                case DiaryUnit.SUGAR  : sugars.addElement(master.getDiary().elementAt(i));  break;
                case DiaryUnit.MENU   : menus.addElement(master.getDiary().elementAt(i));   break;
                case DiaryUnit.COMMENT: comments.addElement(master.getDiary().elementAt(i));break;
            }
        }
        
        Display.getInstance().invokeAndBlock(new Runnable() {
            public void run() {
                // perform IO operation..
                if (!sugars.isEmpty())   shRes = new DoingPOST(master.getUser()).sendDiarySugars(sugars,master);
                if (!comments.isEmpty()) commRes = new DoingPOST(master.getUser()).sendDiaryComments(comments,master);
                if (!menus.isEmpty())    menuRes = new DoingPOST(master.getUser()).sendDiaryMenus(menus,master);
            }
        });
        
        boolean error = (shRes==null?false:(!((String)shRes.get(DoingPOST.ERROR)).equals(DoingPOST.NO_ERROR)) )&& 
                (commRes==null?false:(!((String)commRes.get(DoingPOST.ERROR)).equals(DoingPOST.NO_ERROR)) )    &&
                (menuRes==null?false:(!((String)menuRes.get(DoingPOST.ERROR)).equals(DoingPOST.NO_ERROR)) );
        
        if (!error){
            master.getDiary().removeAllElements();
            master.getDiaryRecs().removeAllElements();
            master.setNeedSaveDiary();
        }
        
        if (!error){
            //тут проверяем версию программы
            ver_check = new DoingPOST(master.getUser()).checkVersion();
        }
        
        Display.getInstance().callSeriallyAndWait(new Runnable(){
            public void run(){
                infoDlg.dispose();
            }
        });
        
        if (!error){
            //Все успешно, показываем поздравление
            TextArea txt = new TextArea("Записи дневника успешно выгружены на сервер");
            txt.setEditable(false);
            Dialog.show("Сообщение",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_INFO,
                            null);
            VersionInfo.showVersionInfo(ver_check);
        }
        else{
            //произошла ошибка
            String msg = shRes==null?"":( ((String)shRes.get(DoingPOST.ERROR)).equals(DoingPOST.NO_ERROR)?
                            "":((String)shRes.get(DoingPOST.ERROR)+"\n") ) +
                         commRes==null?"":( ((String)commRes.get(DoingPOST.ERROR)).equals(DoingPOST.NO_ERROR)?
                            "":((String)commRes.get(DoingPOST.ERROR)+"\n") ) +
                         menuRes==null?"":( ((String)menuRes.get(DoingPOST.ERROR)).equals(DoingPOST.NO_ERROR)?
                            "":((String)menuRes.get(DoingPOST.ERROR)+"\n") );
                    
            TextArea txt = new TextArea("При выгрузке записей дневника произошли ошибки\n"+
                    Str.replace(msg,"<br>","\n"));
            txt.setEditable(false);
            Dialog.show("Ошибка",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_ERROR,
                            null);
        }
        
    }
    private void getInfoPerDay(){
        if (diaryL.getModel().getSize()==0 || 
                diaryL.getSelectedIndex()<0 || 
                diaryL.getSelectedIndex()>(diaryL.getModel().getSize()-1)) return;
        
        DiaryUnit ev = (DiaryUnit)diaryL.getSelectedItem();
        java.util.Calendar c = java.util.Calendar.getInstance();
        c.setTime(new Date(ev.getTime()));
        
        c.set(java.util.Calendar.HOUR_OF_DAY, 0);
        c.set(java.util.Calendar.MINUTE, 0);
        c.set(java.util.Calendar.SECOND, 0);
        c.set(java.util.Calendar.MILLISECOND, 0);
        
        long start_day = c.getTime().getTime();
        long end_day = start_day + 60*60*24*1000;
        
        float dose = 0f;
        float calor = 0f;
        float prot = 0;
        float fat = 0;
        float carb = 0f;
        
        for(int i=0;i<master.getDiary().size();i++){
            DiaryUnit du = (DiaryUnit)master.getDiary().elementAt(i);
            if (du.getTime()>=start_day && du.getTime()<end_day && 
                    du.getType()==DiaryUnit.MENU){
                /*switch (du.getType()){
                    case DiaryUnit.MENU:*/    
                dose += du.getDose();
                calor += du.getProduct().getCalories();
                prot += du.getProduct().getAllProt();
                fat += du.getProduct().getAllFat();
                carb += du.getProduct().getAllCarb();
                    /*break;
                }*/
            }
        }
        
        TextArea txt = new TextArea("В сумме за день\nинсулина: "+ Str.round(dose, 1) + 
                "\nКалорий: "+Str.round(calor, 0) + 
                "\n[Б-Ж-У]\n" + Str.round(prot, 1)+"-"+Str.round(fat, 0)+"-"+Str.round(carb, 0)
                    );
            txt.setEditable(false);
            Dialog.show(""+Str.extendZero(c.get(java.util.Calendar.DAY_OF_MONTH))+"."+
                    Str.extendZero(c.get(java.util.Calendar.MONTH)+1)+"."+
                    c.get(java.util.Calendar.YEAR),
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_ERROR,
                            null);
    }
    private void fillMenu(){
        if (diaryL.size()==0) return;
        if (diaryL.getSelectedIndex()>=0 &&
                ((DiaryUnit)diaryL.getSelectedItem()).getType()==DiaryUnit.MENU
                ){
            TextArea txt = new TextArea("Заполнить меню сохраненной записью?\n" +
                    "Существующее меню будет очищено!");
            txt.setEditable(false);
            if (Dialog.show("Сообщение",
                    txt,
                    new Command[]{yesCommand,noCommand} ).getId()==YES_COMMAND){
                Vector menu = new Vector();
                DiaryUnit du = (DiaryUnit)diaryL.getSelectedItem();
                for (int i=0;i<master.getDiaryRecs().size();i++){
                    if (du.getId() == ((ProductInMenu)master.getDiaryRecs().elementAt(i)).getOwner() ){
                        ProductInMenu pr = (ProductInMenu)master.getDiaryRecs().elementAt(i);
                        menu.addElement(new ProductInMenu(
                                pr.getName(),
                                pr.getProt(),
                                pr.getFat(),
                                pr.getCarb(),
                                pr.getGi(),
                                pr.getWeight(),
                                -1
                                ));
                    }
                }
                master.getUser().setS1(du.getSh1());
                master.getUser().setS2(du.getSh2());
                master.setMenu(menu);
                master.getUser().setFactors( new Factors(du.getFactors()));
            }
        }
    }
    private void deleteEvent(){
        if (diaryL.size()==0) return;
        if (diaryL.getSelectedIndex()>=0){
            DiaryUnit du = (DiaryUnit)diaryL.getSelectedItem();
            java.util.Calendar c = java.util.Calendar.getInstance();
            c.setTime(new java.util.Date(du.getTime()));
            String msg = Str.extendZero(c.get(java.util.Calendar.HOUR_OF_DAY))+":"+
                     Str.extendZero(c.get(java.util.Calendar.MINUTE))+" "+
                     Str.extendZero(c.get(java.util.Calendar.DAY_OF_MONTH))+"."+
                     Str.extendZero(c.get(java.util.Calendar.MONTH)+1)+"."+
                     c.get(java.util.Calendar.YEAR)+" "+
                     du.getComment();
            TextArea txt = new TextArea("Удаляем запись?\n"+msg);
            txt.setEditable(false);
            if (Dialog.show("Удалить запись", txt,
                    new Command[]{yesCommand,noCommand} ).getId()==YES_COMMAND){
                diaryL.getModel().removeItem(diaryL.getModel().getSelectedIndex());
                master.getDiary().removeElement(du);
                if (du.getType()==DiaryUnit.MENU){
                    for(int i=master.getDiaryRecs().size();i>0;i--){
                        if ( ((ProductInMenu)master.getDiaryRecs().elementAt(i-1)).getOwner()==du.getId() ){
                            master.getDiaryRecs().removeElementAt(i-1);
                        }
                    }
                }
                master.setNeedSaveDiary();
            }
        }
    }
    private void editEvent(){
        if (diaryL.size()==0) return;
        if (diaryL.size()>0 && diaryL.getSelectedIndex()>=0){
            DiaryUnit du = (DiaryUnit)diaryL.getModel().getItemAt(diaryL.getSelectedIndex());
            Container c = new EventEditor(du,master.getUser());
            if (YES_COMMAND==Dialog.show("Редакт. запись",
                    c,
                    new Command[]{yesCommand,noCommand} ).getId()){
                DiaryUnit duN = ((EventEditor)c).getDiaryUnit();

                du.setComment(duN.getComment());
                if (du.getType()==DiaryUnit.MENU || du.getType()==DiaryUnit.SUGAR) du.setSh1(duN.getSh1());

                if (du.getType()==DiaryUnit.MENU) du.setSh2(duN.getSh2());
                du.setTime(duN.getTime());
                    
                Ordering.setSortField(Ordering.TIME_FIELD);
                Ordering.setSortType(Ordering.SORT_BY_ASC);
                Ordering.sort(master.getDiary());

                for (int i=0;i<diaryL.size();i++){
                    if ( ((DiaryUnit)diaryL.getModel().getItemAt(i)).getId() == du.getId()){
                        diaryL.setSelectedIndex(i, true);
                        break;
                    }
                }
                master.setNeedSaveDiary();
            }
        }
    }
    private void addEvent(){
        DiaryUnit du = new DiaryUnit(-1,
                System.currentTimeMillis() + master.getUser().getTimeCorr()*1000*60*60,
                "",
                master.getUser().getTargetSugar());
        Container c = new EventEditor(du,master.getUser());
        if (YES_COMMAND==Dialog.show("Новое событие",
                c,
                new Command[]{yesCommand,noCommand} ).getId()){
                du = ((EventEditor)c).getDiaryUnit();
                diaryL.addItem(du);
                Ordering.setSortField(Ordering.TIME_FIELD);
                Ordering.setSortType(Ordering.SORT_BY_ASC);
                Ordering.sort(master.getDiary());

                for (int i=0;i<diaryL.size();i++){
                    if ( ((DiaryUnit)diaryL.getModel().getItemAt(i)).getId() == du.getId()){
                        diaryL.setSelectedIndex(i, true);
                        break;
                    }
                }
                master.setNeedSaveDiary();
        }
    }
    private void addComment(){
        DiaryUnit du = new DiaryUnit(-1,
                System.currentTimeMillis() + master.getUser().getTimeCorr()*1000*60*60,
                "");
        Container c = new EventEditor(du,master.getUser());
        if (YES_COMMAND==Dialog.show("Новое событие",
                c,
                new Command[]{yesCommand,noCommand} ).getId()){
                du = ((EventEditor)c).getDiaryUnit();
                diaryL.addItem(du);
                Ordering.setSortField(Ordering.TIME_FIELD);
                Ordering.setSortType(Ordering.SORT_BY_ASC);
                Ordering.sort(master.getDiary());

                for (int i=0;i<diaryL.size();i++){
                    if ( ((DiaryUnit)diaryL.getModel().getItemAt(i)).getId() == du.getId()){
                        diaryL.setSelectedIndex(i, true);
                        break;
                    }
                }
                master.setNeedSaveDiary();
        }
    }
    private ContextMenu contextMenu = new ContextMenu();
    
    protected void execute(MIDlet m,boolean refresh) {
        if (contextMenu.getCommandsSize()==0){
            contextMenu.addCommand(editEventCommand,this);
            contextMenu.addCommand(fillMenuCommand,this);
            contextMenu.addCommand(delEventCommand,this);
        }
        if (refresh){
            form.getContentPane().setLayout(new BorderLayout());
            master = (DCJmobile)m;
            model = new DefaultListModel(master.getDiary());
            diaryL = new List(model){
                public void longPointerPress(int x, int y){
                    super.longPointerPress(x, y);
                    //Тут показываем контекстное меню.
                    if (contextMenu.getCommandsSize()==0 || 
                            diaryL.getModel().getSize()==0 ||
                            diaryL.getSelectedIndex()<0 || 
                            diaryL.getSelectedIndex()>(diaryL.getModel().getSize())) return;
                    //Container 
                    DiaryUnit du = (DiaryUnit)diaryL.getSelectedItem();
                    String title = "";
                    switch (du.getType()){
                        case DiaryUnit.COMMENT: fillMenuCommand.setEnabled(false);
                                                editEventCommand.setEnabled(true);
                                                title = "Комментарий";
                                                break;
                        case DiaryUnit.MENU   : fillMenuCommand.setEnabled(true);
                                                editEventCommand.setEnabled(true);
                                                title = "Меню";
                                                break;
                        case DiaryUnit.SUGAR  : fillMenuCommand.setEnabled(false);
                                                editEventCommand.setEnabled(true);
                                                title = "Запись о СК";
                                                break;
                    }
                    contextMenu.showContextMenu( title );
                }
            };
            diaryL.setRenderer(new DiaryRenderer());
            
            diaryL.setFireOnClick(false);
            diaryL.setFixedSelection(List.FIXED_NONE_CYCLIC);//двигаемся по кругу "карусель"

            form.getContentPane().addComponent(BorderLayout.CENTER, diaryL);
            
            form.addMenuCommand(newEventCommand,this);
            form.addMenuCommand(newCommentCommand,this);
            form.addMenuCommand(exportDiaryCommand,this);
            form.addMenuCommand(getInfoPerDayCommand, this);
            
            form.getContentPane().setScrollable(false);
        }
    }
    
    public String getName() {
        return "Дневник";
    }

    protected String getHelp() {
        return "";
        /*return "В этом окне вы можете просматривать сохраненные события, " +
                "можно сохранить запись о меню и запись о замере СК.\n" +
                "Не стоит хранить слишком много записей, выгружайте их " +
                "периодически на сервер.\n" +
                "Запись о СК можно добавить из этого окна, а запись о меню добавляется в окне Меню.";*/
    }
    private class DiaryRenderer extends Container implements ListCellRenderer{
        private Label dummy = new Label();
        private java.util.Calendar c = java.util.Calendar.getInstance();
        private Container pane = new Container(new BoxLayout(BoxLayout.Y_AXIS));
        private AnimatedRenderer  top = new AnimatedRenderer (false);
        private AnimatedRenderer bottom = new AnimatedRenderer(false);
        private List parentList;
        private Sugar s1 = new Sugar();
        private Sugar s2 = new Sugar();

        private Label info = new Label();

        private final static int LIGHTGREEN = 0x92F79A;
        private final static int LIGHTBROWN = 0xFFD895;
        private final static int LIGHTCYAN = 0xAEF0F8;
        private final int DEFAULT = UIManager.getInstance().getComponentStyle("Label").getBgColor();
        private final static int WHITE = 0xFFFFFF;

        private String inf;
        private String st2;
                
        DiaryRenderer(){
            super(new BorderLayout());
            this.addComponent("Center",info);

            top.getSelectedStyle().setBgTransparency(0);
            top.setFocus(true);
            top.setFocusable(true);
            top.setTickerEnabled(true);

            bottom.getSelectedStyle().setBgTransparency(0);
            bottom.setFocus(true);
            bottom.setFocusable(true);
            bottom.setTickerEnabled(true);

            pane.addComponent(top);
            pane.addComponent(bottom);

            pane.getStyle().setBorder(Border.createLineBorder(1));
            pane.getStyle().setBgTransparency(255);
            pane.getStyle().setBgColor(WHITE);
            
            info.getStyle().setBgTransparency(127);
        }
        public void restartTicker(){
          top.resetPosition();
          bottom.resetPosition();
        }
        public boolean animate() {
          if(parentList != null && parentList.getComponentForm() != null) {
              if(top.isTickerRunning()) {
                  if(top.animate()) {
                      parentList.repaint();
                  }
              }
              if(bottom.isTickerRunning()) {
                  if(bottom.animate()) {
                      parentList.repaint();
                  }
              }
          }
          return super.animate();
        }
        public Component getListCellRendererComponent(List list, Object value, int index, boolean isSelected){
            DiaryUnit du = (DiaryUnit)value;
            c.setTime(new Date(du.getTime()));
            String st = Str.extendZero(c.get(java.util.Calendar.HOUR_OF_DAY))+":"+
                     Str.extendZero(c.get(java.util.Calendar.MINUTE))+" "+
                     Str.extendZero(c.get(java.util.Calendar.DAY_OF_MONTH))+"."+
                     Str.extendZero(c.get(java.util.Calendar.MONTH)+1)+"."+
                     c.get(java.util.Calendar.YEAR) % 100;//Сформировали дату и время
            int color;
            if (du.getType()==DiaryUnit.COMMENT){//задали цвет фона
                if (isSelected) color = WHITE;
                else color = DEFAULT;
            }
            else color = du.getSh1() < master.getUser().getLowSugar() ? LIGHTCYAN
                    :(du.getSh1()>master.getUser().getHiSugar()?LIGHTBROWN:LIGHTGREEN);
            if (isSelected){
                if (du.getType()==DiaryUnit.MENU){
                    s1.setValue(du.getSh1());
                    s2.setValue(du.getSh2());

                    switch (master.getUser().getInfo()){
                        case User.PFC_INFO: inf = Str.round(du.getProduct().getAllProt(),1)+
                        "-" + Str.round(du.getProduct().getAllFat(),1)+"-"+
                                Str.round(du.getProduct().getAllCarb(),1);
                        break;
                        case User.BE_INFO: inf = "ХЕ="+Str.round(du.getProduct().getAllCarb()/
                                master.getUser().getFactors().getBE(Factors.DIRECT),1);
                        break;
                        case User.CALOR_INFO: inf = "Кал:" + (int)Str.round(du.getProduct().getCalories(),0);
                        break;
                        case User.DOSE_INFO: inf = "Д=" +
                                Str.round(du.getDose()
                                /*new Dose(
                                        du.getProduct(),
                                        du.getFactors(),
                                        new DPS()).getWholeDose(),1)*/,1)+
                                        " "+Str.round(du.getProduct().getAllProt(),1)+
                                        "-" + Str.round(du.getProduct().getAllFat(),1)+
                                        "-"+Str.round(du.getProduct().getAllCarb(),1);
                        break;
                        default: inf = "===";

                    }
                    st2 = st+" СК "+Str.round(s1.getSugar(master.getUser().isMmol(), master.getUser().isPlasma()),1)+
                            " ["+inf+"]";

                }else if (du.getType()==DiaryUnit.SUGAR){
                    s1.setValue(du.getSh1());
                    st2 = st+" СК "+Str.round(s1.getSugar(master.getUser().isMmol(), master.getUser().isPlasma()),1);
                }else /*if (du.getType()==DiaryUnit.COMMENT)*/{
                    st2 = st;
                }
                
                top.getListCellRendererComponent(list, st2, index, isSelected);
                top.setText(st2);
                top.getStyle().setBgColor(0x3ae569);
                bottom.setText(du.getComment());
                if(index == list.getSelectedIndex()) {
                      if(top.shouldTickerStart()) {
                          if(!top.isTickerRunning()) {
                              parentList = list;
                              list.getComponentForm().registerAnimated(this);
                              top.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
                          }
                      } else {
                          if(top.isTickerRunning()) {
                              top.stopTicker();
                          }
                      }
                      if (bottom.shouldTickerStart()){
                          if (!bottom.isTickerRunning()){
                              parentList = list;
                              list.getComponentForm().registerAnimated(this);
                              bottom.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
                          }
                      }
                      else{
                          if(bottom.isTickerRunning()) {
                              bottom.stopTicker();
                          }
                      }
                  }
                 pane.getStyle().setBgColor(color);
                 //pane.getStyle().setBgTransparency(255);
                return pane;
            }
            else{
                if (du.getType()==DiaryUnit.COMMENT){
                    info.setText(st+" "+du.getComment());//тут добавляем СК или коментарий
                }
                else{
                    s1.setValue(du.getSh1());
                    info.setText(st+" СК "+Str.round(s1
                            .getSugar(master.getUser().isMmol(),
                            master.getUser().isPlasma()),1));
                }
                info.getStyle().setBgColor(color);
                
                return this;
            }
        }
        public Component getListFocusComponent(List list) {
            return dummy;
        }
    }
}
